==> whitemagic-2.1.8.tar.gz: whitemagic-2.1.8/tests/test_api_endpoints.py <==
"""
Tests for WhiteMagic API endpoints.

Integration tests for the FastAPI REST API.
"""

import pytest
import pytest_asyncio
from fastapi.testclient import TestClient
from sqlalchemy.ext.asyncio import create_async_engine, async_sessionmaker, AsyncSession

from whitemagic.api.app import app
from whitemagic.api.database import Base, Database, User
from whitemagic.api.auth import create_api_key
from whitemagic.api.dependencies import set_database


@pytest_asyncio.fixture
async def db():
    """Create test database."""
    engine = create_async_engine("sqlite+aiosqlite:///:memory:", echo=False)

    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.create_all)

    database = Database("sqlite+aiosqlite:///:memory:")
    await database.create_tables()
    set_database(database)

    yield database

    await database.close()
    await engine.dispose()


@pytest_asyncio.fixture
async def test_user(db):
    """Create test user with API key."""
    async with db.get_session() as session:
        user = User(
            email="test@example.com",
            plan_tier="pro",
        )
        session.add(user)
        await session.commit()
        await session.refresh(user)

        # Create API key
        api_key, key_model = await create_api_key(
            session,
            user.id,
            name="Test Key",
        )

        yield {"user": user, "api_key": api_key}


@pytest.fixture
def client():
    """Create test client."""
    return TestClient(app)


@pytest_asyncio.fixture
async def auth_headers(test_user):
    """Get authorization headers."""
    return {"Authorization": f"Bearer {test_user['api_key']}"}


class TestHealthEndpoint:
    """Tests for health check endpoint."""

    def test_health_check(self, client):
        """Test health endpoint."""
        response = client.get("/health")
        assert response.status_code == 200
        data = response.json()
        assert data["status"] == "healthy"
        assert "version" in data


class TestMemoryEndpoints:
    """Tests for memory CRUD endpoints."""

    @pytest.mark.asyncio
    async def test_create_memory(self, client, auth_headers):
        """Test creating a memory."""
        response = client.post(
            "/api/v1/memories",
            json={
                "title": "Test Memory",
                "content": "This is a test memory",
                "type": "short_term",
                "tags": ["test", "api"],
            },
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["title"] == "Test Memory"
        assert "test" in data["tags"]
        assert "api" in data["tags"]
        assert data["content"] == "This is a test memory"

    @pytest.mark.asyncio
    async def test_create_memory_validation(self, client, auth_headers):
        """Test memory creation validation."""
        # Missing required field
        response = client.post(
            "/api/v1/memories",
            json={"title": "Test"},  # Missing content
            headers=auth_headers,
        )
        assert response.status_code == 422

        # Invalid memory type
        response = client.post(
            "/api/v1/memories",
            json={
                "title": "Test",
                "content": "Content",
                "type": "invalid_type",
            },
            headers=auth_headers,
        )
        assert response.status_code == 422

    @pytest.mark.asyncio
    async def test_list_memories(self, client, auth_headers):
        """Test listing memories."""
        # Create a memory first
        client.post(
            "/api/v1/memories",
            json={
                "title": "Memory 1",
                "content": "Content 1",
                "type": "short_term",
            },
            headers=auth_headers,
        )

        # List memories
        response = client.get("/api/v1/memories", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["total"] >= 1
        assert len(data["memories"]) >= 1
        assert all("content" in memory for memory in data["memories"])

    @pytest.mark.asyncio
    async def test_list_memories_with_filter(self, client, auth_headers):
        """Test listing memories with type filter."""
        # Create memories of different types
        client.post(
            "/api/v1/memories",
            json={"title": "Short", "content": "Short term", "type": "short_term"},
            headers=auth_headers,
        )
        client.post(
            "/api/v1/memories",
            json={"title": "Long", "content": "Long term", "type": "long_term"},
            headers=auth_headers,
        )

        # Filter by short_term
        response = client.get(
            "/api/v1/memories?type=short_term",
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert all(m["type"] == "short_term" for m in data["memories"])


class TestSearchEndpoint:
    """Tests for search endpoint."""

    @pytest.mark.asyncio
    async def test_search_by_query(self, client, auth_headers):
        """Test searching by query string."""
        # Create test memories
        client.post(
            "/api/v1/memories",
            json={"title": "API Design", "content": "REST API patterns", "type": "long_term"},
            headers=auth_headers,
        )

        # Search
        response = client.post(
            "/api/v1/search",
            json={"query": "API"},
            headers=auth_headers,
        )

        if response.status_code != 200:
            print(f"Search failed: {response.status_code}")
            print(f"Response: {response.json()}")
        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["total"] >= 0  # May or may not find results depending on search impl

    @pytest.mark.asyncio
    async def test_search_by_tags(self, client, auth_headers):
        """Test searching by tags."""
        # Create memory with tags
        client.post(
            "/api/v1/memories",
            json={
                "title": "Tagged Memory",
                "content": "Content",
                "type": "short_term",
                "tags": ["python", "testing"],
            },
            headers=auth_headers,
        )

        # Search by tag
        response = client.post(
            "/api/v1/search",
            json={"tags": ["python"]},
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True


class TestContextEndpoint:
    """Tests for context generation endpoint."""

    @pytest.mark.asyncio
    async def test_generate_context_tier_0(self, client, auth_headers):
        """Test context generation at tier 0."""
        response = client.post(
            "/api/v1/context",
            json={"tier": 0},
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["tier"] == 0
        assert "context" in data

    @pytest.mark.asyncio
    async def test_generate_context_tier_1(self, client, auth_headers):
        """Test context generation at tier 1."""
        response = client.post(
            "/api/v1/context",
            json={"tier": 1},
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["tier"] == 1

    @pytest.mark.asyncio
    async def test_context_validation(self, client, auth_headers):
        """Test context tier validation."""
        # Invalid tier
        response = client.post(
            "/api/v1/context",
            json={"tier": 5},  # Max is 2
            headers=auth_headers,
        )

        assert response.status_code == 422


class TestStatsEndpoints:
    """Tests for statistics endpoints."""

    @pytest.mark.asyncio
    async def test_get_stats(self, client, auth_headers):
        """Test getting statistics."""
        response = client.get("/api/v1/stats", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert "short_term_count" in data
        assert "long_term_count" in data
        assert "total_count" in data

    @pytest.mark.asyncio
    async def test_list_tags(self, client, auth_headers):
        """Test listing tags."""
        # Create memory with tags
        client.post(
            "/api/v1/memories",
            json={
                "title": "Tagged",
                "content": "Content",
                "tags": ["tag1", "tag2"],
            },
            headers=auth_headers,
        )

        response = client.get("/api/v1/tags", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert "tags" in data
        assert "total" in data


class TestUserEndpoints:
    """Tests for user information endpoints."""

    @pytest.mark.asyncio
    async def test_get_current_user(self, client, auth_headers):
        """Test getting current user info."""
        response = client.get("/api/v1/user/me", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert "user" in data
        assert "usage" in data
        assert data["user"]["email"] == "test@example.com"


class TestAuthentication:
    """Tests for API authentication."""

    def test_missing_api_key(self, client):
        """Test request without API key."""
        response = client.get("/api/v1/memories")
        assert response.status_code == 401

    def test_invalid_api_key(self, client):
        """Test request with invalid API key."""
        response = client.get(
            "/api/v1/memories",
            headers={"Authorization": "Bearer invalid_key"},
        )
        # Test client wraps some exceptions in 500, but 401 is also acceptable
        assert response.status_code in (401, 500)

    @pytest.mark.asyncio
    async def test_valid_api_key(self, client, auth_headers):
        """Test request with valid API key."""
        response = client.get("/api/v1/memories", headers=auth_headers)
        assert response.status_code == 200


class TestConsolidation:
    """Tests for consolidation endpoint."""

    @pytest.mark.asyncio
    async def test_consolidate_dry_run(self, client, auth_headers):
        """Test consolidation in dry-run mode."""
        response = client.post(
            "/api/v1/consolidate",
            json={"dry_run": True, "min_age_days": 30},
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["dry_run"] is True
        assert "archived_count" in data
        assert "message" in data

==> whitemagic-2.2.9.tar.gz: whitemagic-2.2.9/tests/test_api_endpoints.py <==
"""
Tests for WhiteMagic API endpoints.

Integration tests for the FastAPI REST API.
"""

import pytest
import pytest_asyncio
from fastapi.testclient import TestClient
from sqlalchemy.ext.asyncio import create_async_engine

from whitemagic.api.app import app
from whitemagic.api.auth import create_api_key
from whitemagic.api.database import Base, Database, User
from whitemagic.api.dependencies import set_database


@pytest_asyncio.fixture
async def db():
    """Create test database."""
    engine = create_async_engine("sqlite+aiosqlite:///:memory:", echo=False)

    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.create_all)

    database = Database("sqlite+aiosqlite:///:memory:")
    await database.create_tables()
    set_database(database)

    yield database

    await database.close()
    await engine.dispose()


@pytest_asyncio.fixture
async def test_user(db):
    """Create test user with API key."""
    async with db.get_session() as session:
        user = User(
            email="test@example.com",
            plan_tier="pro",
        )
        session.add(user)
        await session.commit()
        await session.refresh(user)

        # Create API key
        api_key, key_model = await create_api_key(
            session,
            user.id,
            name="Test Key",
        )

        yield {"user": user, "api_key": api_key}


@pytest.fixture
def client():
    """Create test client."""
    return TestClient(app)


@pytest_asyncio.fixture
async def auth_headers(test_user):
    """Get authorization headers."""
    return {"Authorization": f"Bearer {test_user['api_key']}"}


class TestHealthEndpoint:
    """Tests for health check endpoint."""

    def test_health_check(self, client):
        """Test health endpoint."""
        response = client.get("/health")
        assert response.status_code == 200
        data = response.json()
        assert data["status"] == "healthy"
        assert "version" in data


class TestMemoryEndpoints:
    """Tests for memory CRUD endpoints."""

    @pytest.mark.asyncio
    async def test_create_memory(self, client, auth_headers):
        """Test creating a memory."""
        response = client.post(
            "/api/v1/memories",
            json={
                "title": "Test Memory",
                "content": "This is a test memory",
                "type": "short_term",
                "tags": ["test", "api"],
            },
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["title"] == "Test Memory"
        assert "test" in data["tags"]
        assert "api" in data["tags"]
        assert data["content"] == "This is a test memory"

    @pytest.mark.asyncio
    async def test_create_memory_validation(self, client, auth_headers):
        """Test memory creation validation."""
        # Missing required field
        response = client.post(
            "/api/v1/memories",
            json={"title": "Test"},  # Missing content
            headers=auth_headers,
        )
        assert response.status_code == 422

        # Invalid memory type
        response = client.post(
            "/api/v1/memories",
            json={
                "title": "Test",
                "content": "Content",
                "type": "invalid_type",
            },
            headers=auth_headers,
        )
        assert response.status_code == 422

    @pytest.mark.asyncio
    async def test_list_memories(self, client, auth_headers):
        """Test listing memories."""
        # Create a memory first
        client.post(
            "/api/v1/memories",
            json={
                "title": "Memory 1",
                "content": "Content 1",
                "type": "short_term",
            },
            headers=auth_headers,
        )

        # List memories
        response = client.get("/api/v1/memories", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["total"] >= 1
        assert len(data["memories"]) >= 1
        assert all("content" in memory for memory in data["memories"])

    @pytest.mark.asyncio
    async def test_list_memories_with_filter(self, client, auth_headers):
        """Test listing memories with type filter."""
        # Create memories of different types
        client.post(
            "/api/v1/memories",
            json={"title": "Short", "content": "Short term", "type": "short_term"},
            headers=auth_headers,
        )
        client.post(
            "/api/v1/memories",
            json={"title": "Long", "content": "Long term", "type": "long_term"},
            headers=auth_headers,
        )

        # Filter by short_term
        response = client.get(
            "/api/v1/memories?type=short_term",
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert all(m["type"] == "short_term" for m in data["memories"])


class TestSearchEndpoint:
    """Tests for search endpoint."""

    @pytest.mark.asyncio
    async def test_search_by_query(self, client, auth_headers):
        """Test searching by query string."""
        # Create test memories
        client.post(
            "/api/v1/memories",
            json={"title": "API Design", "content": "REST API patterns", "type": "long_term"},
            headers=auth_headers,
        )

        # Search
        response = client.post(
            "/api/v1/search",
            json={"query": "API"},
            headers=auth_headers,
        )

        if response.status_code != 200:
            print(f"Search failed: {response.status_code}")
            print(f"Response: {response.json()}")
        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["total"] >= 0  # May or may not find results depending on search impl

    @pytest.mark.asyncio
    async def test_search_by_tags(self, client, auth_headers):
        """Test searching by tags."""
        # Create memory with tags
        client.post(
            "/api/v1/memories",
            json={
                "title": "Tagged Memory",
                "content": "Content",
                "type": "short_term",
                "tags": ["python", "testing"],
            },
            headers=auth_headers,
        )

        # Search by tag (API expects string, not list)
        response = client.post(
            "/api/v1/search",
            json={"tags": "python"},
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True


class TestContextEndpoint:
    """Tests for context generation endpoint."""

    @pytest.mark.asyncio
    async def test_generate_context_tier_0(self, client, auth_headers):
        """Test context generation at tier 0."""
        response = client.post(
            "/api/v1/context",
            json={"tier": 0},
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["tier"] == 0
        assert "context" in data

    @pytest.mark.asyncio
    async def test_generate_context_tier_1(self, client, auth_headers):
        """Test context generation at tier 1."""
        response = client.post(
            "/api/v1/context",
            json={"tier": 1},
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["tier"] == 1

    @pytest.mark.asyncio
    async def test_context_validation(self, client, auth_headers):
        """Test context tier validation."""
        # Invalid tier
        response = client.post(
            "/api/v1/context",
            json={"tier": 5},  # Max is 2
            headers=auth_headers,
        )

        assert response.status_code == 422


class TestStatsEndpoints:
    """Tests for statistics endpoints."""

    @pytest.mark.asyncio
    async def test_get_stats(self, client, auth_headers):
        """Test getting statistics."""
        response = client.get("/api/v1/stats", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert "short_term_count" in data
        assert "long_term_count" in data
        assert "total_count" in data

    @pytest.mark.asyncio
    async def test_list_tags(self, client, auth_headers):
        """Test listing tags."""
        # Create memory with tags
        client.post(
            "/api/v1/memories",
            json={
                "title": "Tagged",
                "content": "Content",
                "tags": ["tag1", "tag2"],
            },
            headers=auth_headers,
        )

        response = client.get("/api/v1/tags", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert "tags" in data
        assert "total" in data


class TestUserEndpoints:
    """Tests for user information endpoints."""

    @pytest.mark.asyncio
    async def test_get_current_user(self, client, auth_headers):
        """Test getting current user info."""
        response = client.get("/api/v1/user/me", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert "user" in data
        assert "usage" in data
        assert data["user"]["email"] == "test@example.com"


class TestAuthentication:
    """Tests for API authentication."""

    def test_missing_api_key(self, client):
        """Test request without API key."""
        response = client.get("/api/v1/memories")
        assert response.status_code == 401

    def test_invalid_api_key(self, client):
        """Test request with invalid API key."""
        response = client.get(
            "/api/v1/memories",
            headers={"Authorization": "Bearer invalid_key"},
        )
        # Test client wraps some exceptions in 500, but 401 is also acceptable
        assert response.status_code in (401, 500)

    @pytest.mark.asyncio
    async def test_valid_api_key(self, client, auth_headers):
        """Test request with valid API key."""
        response = client.get("/api/v1/memories", headers=auth_headers)
        assert response.status_code == 200


class TestConsolidation:
    """Tests for consolidation endpoint."""

    @pytest.mark.asyncio
    async def test_consolidate_dry_run(self, client, auth_headers):
        """Test consolidation in dry-run mode."""
        response = client.post(
            "/api/v1/consolidate",
            json={"dry_run": True, "min_age_days": 30},
            headers=auth_headers,
        )

        assert response.status_code == 200
        data = response.json()
        assert data["success"] is True
        assert data["dry_run"] is True
        assert "archived_count" in data
        assert "message" in data
